SystemOrganization addCategory: #'AntSimulation-Test'!
SystemOrganization addCategory: #AntSimulation!

TestCase subclass: #EnvironmentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!EnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/3/2012 23:37'!
testAddEntity
	|env entity cell entityStatus|
	env := Environment new initWithRows: 10 columns: 10.

	entity := Entity new.
	cell := env cellAtRow: 3 column: 2.
	env add: entity to: cell.
	
	self assert: env entities size equals: 1.

	entityStatus := env statusForEntity: entity.
	self assert: entityStatus health equals: 1.0 .
	
	self assert: (entityStatus cell) == cell.
	self assert: (entityStatus cell) row equals: 3.
	self assert: (entityStatus cell) column equals: 2.! !

!EnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/3/2012 23:32'!
testCreation
	|env|
	env := Environment new initWithRows: 10 columns: 15.
	self assert: env rowCount equals: 10.
	self assert: env columnCount equals: 15.
	self assert: env entities size equals: 0.
! !

TestCase subclass: #SurfaceEnvironmentTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation-Test'!

!SurfaceEnvironmentTest methodsFor: 'testing' stamp: 'VanceThornton 7/4/2012 00:27'!
testGenerateRandom
	|env|
	env := SurfaceEnvironment generateRandomWithRows: 25 columns: 25 antCount: 10.
	
	self assert: (env entities) size equals: 10.
	
	(env entities) do: [:entity |
		|entityStatus cell|
		entityStatus := env statusForEntity: entity.
		self assert: entityStatus ~= nil.
		
		cell := entityStatus cell.
		self assert: ((cell entities) includes: entity).		
		self assert: (env cellAtRow: (cell row) column: (cell column)) == cell.
	].! !

Object subclass: #AntSensoryInput
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Object subclass: #AntSimulation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

AntSimulation class
	instanceVariableNames: 'random'!

AntSimulation class
	instanceVariableNames: 'random'!

!AntSimulation class methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/4/2012 00:09'!
random
	(random == nil) ifTrue: [random := Random seed: 1].
	^random! !

Object subclass: #Cell
	instanceVariableNames: 'row column entities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Cell methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/3/2012 23:03'!
add: entity
	entities add: entity! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:38'!
column
	^column! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/4/2012 00:25'!
entities
	^entities! !

!Cell methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:24'!
initWithRow: newRow column: newColumn
 	row := newRow.
	column := newColumn.
	entities := Set new.! !

!Cell methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:38'!
row
	^row! !

Object subclass: #Entity
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Entity subclass: #ActiveEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

ActiveEntity subclass: #Ant
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Entity methodsFor: 'comparing' stamp: 'VanceThornton 7/3/2012 18:58'!
= anEntity
	self == anEntity ifTrue: [^true].
	^(id = anEntity id)
! !

!Entity methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:58'!
id
	^id! !

!Entity methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:13'!
initialize
	id := UUID new.! !

Entity subclass: #InactiveEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

InactiveEntity subclass: #Food
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

Object subclass: #EntityStatus
	instanceVariableNames: 'cell health'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:36'!
cell
	^cell! !

!EntityStatus methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:35'!
health
	^health! !

!EntityStatus methodsFor: 'initialize-release' stamp: 'VanceThornton 7/3/2012 23:14'!
initWithCell: aCell
	cell := aCell.
	health := 1.0.
	! !

Object subclass: #Environment
	instanceVariableNames: 'cellMatrix entityStatusMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!Environment methodsFor: 'as yet unclassified' stamp: 'VanceThornton 7/3/2012 23:32'!
add: entity to: cell
	|status|
	status := EntityStatus new initWithCell: cell.
	entityStatusMap add: (Association key: entity value: status).
	cell add: entity.! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 00:56'!
cellAtRow: row column: column
	^cellMatrix at: row at: column
	! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:40'!
columnCount
	^cellMatrix columnCount! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:15'!
entities
	^entityStatusMap keys! !

!Environment methodsFor: 'initialize-release' stamp: 'VanceThornton 7/4/2012 00:06'!
initWithRows: rowCount columns: columnCount
	cellMatrix := Matrix rows: rowCount columns: columnCount.
	cellMatrix indicesDo: [:row :column| cellMatrix at: row at: column put: (Cell new initWithRow: row column: column)].
	entityStatusMap := Dictionary new.! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 18:40'!
rowCount
	^cellMatrix rowCount! !

!Environment methodsFor: 'accessing' stamp: 'VanceThornton 7/3/2012 23:16'!
statusForEntity: entity
	^entityStatusMap at: entity! !

Environment subclass: #SurfaceEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AntSimulation'!

!SurfaceEnvironment class methodsFor: 'class initialization' stamp: 'VanceThornton 7/4/2012 00:14'!
generateRandomWithRows: rows 
	columns: columns 
	antCount: antCount
	
	|env|

	env := SurfaceEnvironment new initWithRows: rows columns: columns.
	
	antCount timesRepeat: [
		|ant cell row column|
		ant := Ant new.
		row := AntSimulation random nextInt: rows.
		column := AntSimulation random nextInt: columns.
		cell := env cellAtRow: row column: column.
		env add: ant to: cell.
	].
	^env! !
